{+--------------------------------------------------------------------------+
 | Class:       TIDEStream
 | Created:     9.97
 | Author:      Martin Waldenburg
 | Copyright    1997, 1998 all rights reserved.
 | Description: A simple but effective interface to the IDE's text buffer
 |              You can retrive the text of the IDE's text buffer either
 |              as PChar or as string list.
 |              You can manipulate it in any way and then replace the
 |              whole text with the result of your manipulation.
 | Version:     1.2
 | Status:      FreeWare
 | Disclaimer:
 | This is provided as is, expressly without a warranty of any kind.
 | You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit mwIDEStream;

interface

uses
  Windows,
  SysUtils,
  Messages,
  Classes,
  ToolIntf,
  EditIntf,
  ExptIntf;

type
  TIDEStream = class(TMemoryStream)
  private
    fLines: TStringList;
    function GetAsPChar: PChar;
    function GetStreamTextLen: Longint;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure WriteText(Text: PChar);
    property Capacity;
    property AsPChar: PChar read GetAsPChar;
    function GetText: PChar;
    procedure SetMemPointer(Ptr: Pointer; Size: Longint);
    procedure GetLines(newValue: TStringList);
    property StreamTextLen: Longint read GetStreamTextLen;
  published
  end;

var
  fToolServices: TIToolServices;
  fModuleInterface: TIModuleInterface;
  fEditorInterface: TIEditorInterface;
  ActualReader: TIEditReader;
  ActualWriter: TIEditWriter;

implementation

function GetProjectName: String;
begin
  Result := fToolServices.GetProjectName;
end; { GetProjectName }

function GetCurrentFile: String;
begin
  Result := fToolServices.GetCurrentFile;
end; { GetCurrentFile }

function GetToolServieces: TIToolServices;
var
  FileExt: String;
begin
  fToolServices := ExptIntf.ToolServices;
  if GetProjectName = '' then raise exception.create('Sorry, a project must be open');
  FileExt := ExtractFileExt(GetCurrentFile);
  if FileExt = '.dfm' then raise exception.create('Sorry, must be a PAS or DPR file');

end; { GetToolServieces }

procedure GetEditReader;
begin
  GetToolServieces;
  fModuleInterface := fToolServices.GetModuleInterface(GetCurrentFile);
  fEditorInterface := fModuleInterface.GetEditorInterface;
  ActualReader := fEditorInterface.CreateReader;
end; { GetEditReader }

procedure GetEditWriter;
begin
  GetToolServieces;
  fModuleInterface := fToolServices.GetModuleInterface(GetCurrentFile);
  fEditorInterface := fModuleInterface.GetEditorInterface;
  ActualWriter := fEditorInterface.CreateWriter;
end; { GetEditWriter }

procedure FreeEditReader;
begin
  ActualReader.Free;
  fEditorInterface.Free;
  fModuleInterface.Free;
end; { GetEditorInterface }

procedure FreeEditWriter;
begin
  ActualWriter.Free;
  fEditorInterface.Free;
  fModuleInterface.Free;
end; { GetEditorInterface }

constructor TIDEStream.Create;
begin
  inherited Create;
end; { Create }

destructor TIDEStream.Destroy;
begin
  inherited Destroy;
end; { Destroy }

function TIDEStream.GetAsPChar: PChar;
const
  TheEnd: Char = #0;
begin
  Position := Size;
  Write(TheEnd, 1);
  SetPointer(Memory, Size - 1);
  Result := Memory;
end; { GetAsPChar }

procedure TIDEStream.SetMemPointer(Ptr: Pointer; Size: Longint);
begin
  SetPointer(Ptr, Size);
end; { SetMemPointer }

function TIDEStream.GetStreamTextLen: Longint;
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos, TextLen: LongInt;
begin
  TextLen := 0;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  GetEditReader;
  try
    repeat
      Readed := ActualReader.GetText(BuffPos, TextBuffer, Bufflen);
      TextLen := TextLen + Readed;
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeEditReader;
    FreeMem(TextBuffer, BuffLen + 1);
  end;
  Result := TextLen;
end; { GetStreamTextLen }

function TIDEStream.GetText: PChar;
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos: LongInt;
begin
  Clear;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  GetEditReader;
  try
    repeat
      Readed := ActualReader.GetText(BuffPos, TextBuffer, Bufflen);
      Write(TextBuffer^, Readed);
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeEditReader;
    FreeMem(TextBuffer, BuffLen + 1);
  end;
  Result := AsPchar;
end; { GetText }

procedure TIDEStream.WriteText(Text: PChar);
var
  TextLen: LongInt;
begin
  TextLen := GetStreamTextLen;
  GetEditWriter;
  try
    ActualWriter.CopyTo(0);
    ActualWriter.DeleteTo(TextLen - 1);
    {For Delphi 2 use: ActualWriter.DeleteTo(TextLen - 2);}
    ActualWriter.Insert(Text);
  finally
    FreeEditWriter;
  end;
end; { WriteText }

procedure TIDEStream.GetLines(newValue: TStringList);
begin
  FLines := NewValue;
  if assigned(FLines) then
  begin
    FLines.Clear;
    Flines.SetText(GetText);
  end;
end; { GetLines }

end.


